@@ -6,9 +6,21 @@
 //! This crate contains utility traits, functions, and macros used by other crates of Winterfell
 //! STARK prover and verifier.
 
+#![cfg_attr(not(feature = "std"), no_std)]
+
+#[cfg(not(feature = "std"))]
+#[macro_use]
+extern crate alloc;
+
 use core::{convert::TryInto, mem, slice};
 
-mod iterators;
+pub mod collections;
+use collections::Vec;
+
+pub mod string;
+use string::ToString;
+
+pub mod iterators;
 
 mod errors;
 pub use errors::DeserializationError;
@@ -17,7 +29,10 @@ pub use errors::DeserializationError;
 mod tests;
 
 #[cfg(feature = "concurrent")]
-use rayon::prelude::*;
+use iterators::*;
+
+#[cfg(feature = "concurrent")]
+pub use rayon;
 
 // SERIALIZABLE
 // ================================================================================================
@@ -130,7 +145,7 @@ pub trait Deserializable: Sized {
     /// * Bytes read from the `source` do not represent a valid value for `Self` for any of the
     ///   elements.
     ///
-    /// Note: if the error occurs, the reader is not rolled back to the state prior calling
+    /// Note: if the error occurs, the reader is not rolled back to the state prior to calling
     /// this function.
     fn read_batch_from<R: ByteReader>(
         source: &mut R,
@@ -418,6 +433,7 @@ impl<const N: usize> AsBytes for [[u8; N]] {
 ///
 /// # Safety
 /// Using values from the returned vector before initializing them will lead to undefined behavior.
+#[allow(clippy::uninit_vec)]
 pub unsafe fn uninit_vector<T>(length: usize) -> Vec<T> {
     let mut vector = Vec::with_capacity(length);
     vector.set_len(length);
@@ -525,6 +541,8 @@ pub fn flatten_vector_elements<T, const N: usize>(source: Vec<[T; N]>) -> Vec<T>
 
 /// Transposes a slice of `n` elements into a matrix with `N` columns and `n`/`N` rows.
 ///
+/// When `concurrent` feature is enabled, the slice will be transposed using multiple threads.
+///
 /// # Panics
 /// Panics if `n` is not divisible by `N`.
 ///
@@ -556,3 +574,74 @@ pub fn transpose_slice<T: Copy + Send + Sync, const N: usize>(source: &[T]) -> V
         });
     result
 }
+
+// RANDOMNESS
+// ================================================================================================
+
+/// Defines how `Self` can be read from a sequence of random bytes.
+pub trait Randomizable: Sized {
+    /// Size of `Self` in bytes.
+    ///
+    /// This is used to determine how many bytes should be passed to the
+    /// [from_random_bytes()](Self::from_random_bytes) function.
+    const VALUE_SIZE: usize;
+
+    /// Returns `Self` if the set of bytes forms a valid value, otherwise returns None.
+    fn from_random_bytes(source: &[u8]) -> Option<Self>;
+}
+
+impl Randomizable for u128 {
+    const VALUE_SIZE: usize = 16;
+
+    fn from_random_bytes(source: &[u8]) -> Option<Self> {
+        if let Ok(bytes) = source[..Self::VALUE_SIZE].try_into() {
+            Some(u128::from_le_bytes(bytes))
+        } else {
+            None
+        }
+    }
+}
+
+impl Randomizable for u64 {
+    const VALUE_SIZE: usize = 8;
+
+    fn from_random_bytes(source: &[u8]) -> Option<Self> {
+        if let Ok(bytes) = source[..Self::VALUE_SIZE].try_into() {
+            Some(u64::from_le_bytes(bytes))
+        } else {
+            None
+        }
+    }
+}
+
+impl Randomizable for u32 {
+    const VALUE_SIZE: usize = 4;
+
+    fn from_random_bytes(source: &[u8]) -> Option<Self> {
+        if let Ok(bytes) = source[..Self::VALUE_SIZE].try_into() {
+            Some(u32::from_le_bytes(bytes))
+        } else {
+            None
+        }
+    }
+}
+
+impl Randomizable for u16 {
+    const VALUE_SIZE: usize = 2;
+
+    fn from_random_bytes(source: &[u8]) -> Option<Self> {
+        if let Ok(bytes) = source[..Self::VALUE_SIZE].try_into() {
+            Some(u16::from_le_bytes(bytes))
+        } else {
+            None
+        }
+    }
+}
+
+impl Randomizable for u8 {
+    const VALUE_SIZE: usize = 1;
+
+    fn from_random_bytes(source: &[u8]) -> Option<Self> {
+        Some(source[0])
+    }
+}
